import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';
import 'package:senior_design_system/senior_design_system.dart';

class DismissibleAction {
  final Color backgroundColor;
  final IconData icon;
  final Function() onDismissed;

  const DismissibleAction({
    required this.backgroundColor,
    required this.icon,
    required this.onDismissed,
  });
}

class DismissibleConfig {
  final Key key;
  final DismissibleAction primaryAction;
  final DismissibleAction? secondaryAction;

  const DismissibleConfig({
    required this.key,
    required this.primaryAction,
    this.secondaryAction,
  });
}

class SeniorCard extends StatelessWidget {
  final Widget content;
  final Function()? pushAction;
  final Color? severityColor;
  final DismissibleConfig? dismissibleConfig;

  const SeniorCard({
    Key? key,
    required this.content,
    this.pushAction,
    this.severityColor,
    this.dismissibleConfig,
  }) : super(key: key);

  factory SeniorCard.checkbox({
    Key key,
    required Widget content,
    required bool value,
    Function() pushAction,
    Color? severityColor,
    DismissibleConfig? dismissibleConfig,
    required Function(bool?) onCheck,
  }) = _SeniorCardCheckbox;

  Widget buildCardContent() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        content,
        Visibility(
          visible: pushAction != null,
          child: Icon(
            FontAwesomeIcons.chevronRight,
            color: SeniorColors.neutralColor600,
            size: SeniorTypographyIconSize.body,
          ),
        ),
      ],
    );
  }

  Widget buildCard() {
    return SeniorElevatedElement(
      borderRadius: SeniorRadius.xbig,
      elevation: SeniorElevations.dp01,
      child: Material(
        color: Colors.transparent,
        child: InkWell(
          borderRadius: BorderRadius.circular(SeniorRadius.xbig),
          onTap: pushAction,
          child: Ink(
            padding: EdgeInsets.only(left: SeniorSpacing.small),
            decoration: BoxDecoration(
              color: severityColor ?? SeniorColors.neutralColor100,
              borderRadius: BorderRadius.circular(SeniorRadius.xbig),
            ),
            child: Ink(
              padding: const EdgeInsets.only(
                top: SeniorSpacing.normal,
                bottom: SeniorSpacing.normal,
                left: SeniorSpacing.xsmall,
                right: SeniorSpacing.normal,
              ),
              decoration: BoxDecoration(
                borderRadius: BorderRadius.only(
                  topRight: Radius.circular(SeniorRadius.xbig),
                  bottomRight: Radius.circular(SeniorRadius.xbig),
                ),
                color: SeniorColors.neutralColor100,
              ),
              child: buildCardContent(),
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return (dismissibleConfig != null)
        ? Dismissible(
            key: dismissibleConfig!.key,
            onDismissed: (direction) {
              if(direction == DismissDirection.startToEnd) {
                dismissibleConfig!.primaryAction.onDismissed();
              } else if(direction == DismissDirection.endToStart) {
                dismissibleConfig!.secondaryAction != null
                    ? dismissibleConfig!.secondaryAction!.onDismissed()
                    : dismissibleConfig!.primaryAction.onDismissed();
              }
            },
            direction: dismissibleConfig!.secondaryAction != null
                ? DismissDirection.horizontal
                : DismissDirection.startToEnd,
            background: dismissibleConfig!.primaryAction != null
                ? Container(
                    decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(SeniorRadius.xbig),
                      color: dismissibleConfig!.primaryAction.backgroundColor,
                    ),
                    child: Padding(
                      padding: EdgeInsets.symmetric(
                        horizontal: SeniorSpacing.medium,
                      ),
                      child: Align(
                        alignment: Alignment.centerLeft,
                        child: FaIcon(
                          dismissibleConfig!.primaryAction.icon,
                          color: SeniorColors.pureWhite,
                          size: SeniorTypographyIconSize.h4,
                        ),
                      ),
                    ),
                  )
                : null,
            secondaryBackground: dismissibleConfig!.secondaryAction != null
                ? Container(
                    decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(SeniorRadius.xbig),
                      color: dismissibleConfig!.secondaryAction!.backgroundColor,
                    ),
                    child: Padding(
                      padding: EdgeInsets.symmetric(
                        horizontal: SeniorSpacing.medium,
                      ),
                      child: Align(
                        alignment: Alignment.centerRight,
                        child: FaIcon(
                          dismissibleConfig!.secondaryAction!.icon,
                          color: SeniorColors.pureWhite,
                          size: SeniorTypographyIconSize.h4,
                        ),
                      ),
                    ),
                  )
                : null,
            child: buildCard(),
          )
        : buildCard();
  }
}

class _SeniorCardCheckbox extends SeniorCard {
  final bool value;
  final Function(bool?) onCheck;

  _SeniorCardCheckbox({
    Key? key,
    required Widget content,
    required this.value,
    Function()? pushAction,
    Color? severityColor,
    DismissibleConfig? dismissibleConfig,
    required this.onCheck,
  }) : super(
          key: key,
          content: content,
          pushAction: pushAction,
          severityColor: severityColor,
          dismissibleConfig: dismissibleConfig,
        );

  @override
  Widget buildCardContent() {
    return Container(
      child: Stack(
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              content,
              Visibility(
                visible: pushAction != null,
                child: Icon(
                  FontAwesomeIcons.chevronRight,
                  color: SeniorColors.neutralColor600,
                  size: SeniorTypographyIconSize.body,
                ),
              ),
            ],
          ),
          Align(
            alignment: Alignment.topRight,
            child: SeniorCheckbox(
              value: value,
              onChanged: onCheck,
            ),
          ),
        ],
      ),
    );
  }
}
