import 'package:flutter/material.dart';
import 'package:senior_design_system/senior_design_system.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';
import 'package:senior_design_tokens/tokens/senior_spacing.dart';

class SeniorNotification {
  final String title;
  final String body;
  final String footer;
  final bool isRead;

  const SeniorNotification({
    required this.title,
    required this.body,
    required this.footer,
    required this.isRead,
  });
}

class SeniorNotificationList extends StatelessWidget {
  final String title;
  final List<SeniorNotification> notifications;
  final Function(int)? onNotificationTap;
  final String? actionButtonLabel;
  final Function()? onTapActionButton;
  final bool isLoading;
  final String? loadButtonLabel;
  final Function()? onLoad;
  final double horizontalPadding;
  final ScrollController? scrollController;
  final bool busyActionButton;
  final bool disabledActionButton;

  SeniorNotificationList({
    Key? key,
    required this.title,
    required this.notifications,
    this.onNotificationTap,
    this.actionButtonLabel,
    this.onTapActionButton,
    this.isLoading = false,
    this.loadButtonLabel,
    this.onLoad,
    this.horizontalPadding = 0,
    this.scrollController,
    this.busyActionButton = false,
    this.disabledActionButton = false,
  }) : super(key: key);

  Widget _buildNotification(SeniorNotification notification) {
    return InkWell(
      child: Container(
        padding: EdgeInsets.symmetric(
          horizontal: horizontalPadding,
          vertical: 16.0,
        ),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Padding(
              padding: const EdgeInsets.only(
                bottom: SeniorSpacing.xsmall,
              ),
              child: Text(
                notification.title,
                style: SeniorTypography.body(
                  color: SeniorColors.pureBlack,
                ).copyWith(
                  fontWeight:
                      notification.isRead ? FontWeight.normal : FontWeight.bold,
                ),
              ),
            ),
            Padding(
              padding: const EdgeInsets.only(
                bottom: SeniorSpacing.xsmall,
              ),
              child: Text(
                notification.body,
                style: SeniorTypography.body(
                  color: SeniorColors.neutralColor800,
                ),
              ),
            ),
            Text(
              notification.footer,
              style: SeniorTypography.helper(
                color: SeniorColors.neutralColor500,
              ),
            ),
          ],
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final padding = MediaQuery.of(context).padding;

    return Container(
      child: Column(
        children: [
          Align(
            alignment: Alignment.centerLeft,
            child: Padding(
              padding: const EdgeInsets.symmetric(
                horizontal: SeniorSpacing.xmedium,
                vertical: SeniorSpacing.small,
              ),
              child: Text(
                title,
                style: SeniorTypography.body(
                  color: SeniorColors.neutralColor800,
                ),
              ),
            ),
          ),
          Expanded(
            child: Scrollbar(
              controller: scrollController,
              child: ListView.separated(
                padding: EdgeInsets.zero,
                controller: scrollController,
                scrollDirection: Axis.vertical,
                shrinkWrap: true,
                itemCount: notifications.length + 1,
                separatorBuilder: (context, index) {
                  return Container(
                    margin: EdgeInsets.symmetric(
                      horizontal: SeniorSpacing.normal,
                    ),
                    height: 2.0,
                    color: SeniorColors.neutralColor300,
                  );
                },
                itemBuilder: (context, index) {
                  if (index == notifications.length) {
                    if (isLoading) {
                      return Padding(
                        padding: const EdgeInsets.all(
                          SeniorSpacing.normal,
                        ),
                        child: Center(
                          child: SeniorLoading(),
                        ),
                      );
                    }

                    return onLoad != null &&
                            loadButtonLabel != null &&
                            !isLoading
                        ? Padding(
                            padding: const EdgeInsets.all(16.0),
                            child: SeniorButton(
                              label: loadButtonLabel!,
                              type: SeniorButtonType.ghost,
                              size: SeniorButtonSize.small,
                              onPressed: onLoad!,
                            ),
                          )
                        : const SizedBox.shrink();
                  }
                  return InkWell(
                    onTap: onNotificationTap != null
                        ? () => onNotificationTap!(index)
                        : null,
                    child: _buildNotification(notifications[index]),
                  );
                },
              ),
            ),
          ),
          (onTapActionButton != null && actionButtonLabel != null)
              ? Padding(
                  padding: const EdgeInsets.all(SeniorSpacing.normal),
                  child: SeniorButton(
                    label: actionButtonLabel!,
                    type: SeniorButtonType.ghost,
                    onPressed: onTapActionButton!,
                    fullLength: true,
                    busy: busyActionButton,
                    disabled: disabledActionButton,
                  ),
                )
              : const SizedBox.shrink(),
          SizedBox(
            height: padding.bottom,
          ),
        ],
      ),
    );
  }
}
