import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

class SeniorTextField extends StatefulWidget {
  final String? label;
  final IconData? leftIcon;
  final IconData? rightIcon;
  final String? helper;
  final String? hintText;
  final bool disabled;
  final String? initialValue;
  final FormFieldValidator<String>? validator;
  final int? maxLength;
  final int? minLines;
  final int? maxLines;
  final bool autocorrect;
  final bool readOnly;
  final Function(String)? onChanged;
  final Function()? onTap;
  final Function()? onEditingComplete;
  final Function(String?)? onFieldSubmitted;
  final bool autofocus;
  final TextEditingController? controller;
  final TextInputType? keyboardType;
  final TextInputAction? textInputAction;
  final List<TextInputFormatter> inputFormatters;
  final String? counterCaracterLabel;

  const SeniorTextField({
    Key? key,
    this.label,
    this.leftIcon,
    this.rightIcon,
    this.helper,
    this.hintText,
    this.disabled = false,
    this.initialValue,
    this.validator,
    this.maxLength,
    this.minLines,
    this.maxLines = 1,
    this.autocorrect = true,
    this.readOnly = false,
    this.onChanged,
    this.onTap,
    this.onEditingComplete,
    this.onFieldSubmitted,
    this.autofocus = false,
    this.controller,
    this.keyboardType,
    this.inputFormatters = const [],
    this.textInputAction,
    this.counterCaracterLabel,
  }) : super(key: key);

  factory SeniorTextField.password({
    Key? key,
    String? label,
    IconData? leftIcon,
    String? helper,
    String? hintText,
    bool disabled,
    String? initialValue,
    FormFieldValidator<String>? validator,
    int? maxLength,
    bool autocorrect,
    bool readOnly,
    Function(String)? onChanged,
    Function()? onTap,
    Function()? onEditingComplete,
    bool autofocus,
    TextEditingController? controller,
    TextInputAction? textInputAction,
  }) = _SeniorTextFieldPassword;

  @override
  _SeniorTextField createState() => _SeniorTextField();
}

class _SeniorTextField extends State<SeniorTextField> {
  late TextEditingController controller;
  late FocusNode focusNode;
  String? validMsg;
  bool hasFocus = false;
  bool hasError = false;
  String? errorMsg;

  @override
  void initState() {
    super.initState();
    focusNode = FocusNode();
    focusNode.addListener(onFocusChange);
    controller = widget.controller ?? TextEditingController();
    internalValidate(value: widget.initialValue, validateAsError: false);
  }

  @override
  void dispose() {
    focusNode.dispose();
    super.dispose();
  }

  void onFocusChange() {
    setState(() {
      hasFocus = focusNode.hasFocus;
    });

    if (!hasFocus) {
      internalValidate(value: controller.text);
    }
  }

  void internalValidate({String? value, bool validateAsError = true}) {
    if (widget.validator != null) {
      validMsg = widget.validator!(value);

      if (validateAsError) {
        hasError = validMsg != null;
        errorMsg = validMsg;
      }
    }
    setState(() {});
  }

  String? validate(String? value) {
    internalValidate(value: value);
    return validMsg;
  }

  void onChanged(String value) {
    internalValidate(value: value);

    if (widget.onChanged != null) {
      return widget.onChanged!(value);
    }
  }

  Widget? buildFaIcon(IconData? icon) {
    if (icon != null) {
      return Padding(
        padding: const EdgeInsets.symmetric(
          horizontal: 12.0,
          vertical: 16.0,
        ),
        child: FaIcon(
          icon,
          size: SeniorTypographyIconSize.CTA,
          color: SeniorColors.neutralColor900,
        ),
      );
    }
    return null;
  }

  Widget buildTextField() {
    var formatters = [
      ...widget.inputFormatters,
    ];

    if (widget.maxLength != null) {
      formatters.add(
        LengthLimitingTextInputFormatter(
          widget.maxLength,
        ),
      );
    }

    return AnimatedContainer(
      duration: const Duration(milliseconds: 200),
      decoration: BoxDecoration(
        color: hasFocus ? SeniorColors.neutralColor100 : Colors.transparent,
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(SeniorRadius.xbig),
          topRight: Radius.circular(SeniorRadius.xbig),
          bottomRight: Radius.circular(SeniorRadius.xsmall),
          bottomLeft: Radius.circular(SeniorRadius.xsmall),
        ),
        border: Border.all(
          color: hasFocus
              ? hasError
                  ? SeniorColors.manchesterColorRed
                  : SeniorColors.primaryColor
              : Colors.transparent,
          width: 2.0,
        ),
      ),
      child: TextFormField(
        key: widget.key,
        textInputAction: widget.textInputAction,
        onFieldSubmitted: widget.onFieldSubmitted,
        style: SeniorTypography.text(color: SeniorColors.neutralColor900),
        onChanged: onChanged,
        onTap: widget.onTap,
        validator: validate,
        focusNode: focusNode,
        minLines: widget.minLines,
        inputFormatters: formatters,
        maxLines: widget.maxLines,
        autocorrect: widget.autocorrect,
        readOnly: widget.readOnly,
        onEditingComplete: widget.onEditingComplete,
        autofocus: widget.autofocus,
        enabled: !widget.disabled,
        controller: controller,
        keyboardType: widget.keyboardType,
        decoration: InputDecoration(
          alignLabelWithHint: widget.maxLines != 1 && widget.maxLines != null,
          labelText: widget.label,
          labelStyle: hasFocus
              ? SeniorTypography.title(
                  color: hasError
                      ? SeniorColors.manchesterColorRed
                      : SeniorColors.primaryColor500,
                )
              : SeniorTypography.text(
                  color: hasError
                      ? SeniorColors.manchesterColorRed
                      : SeniorColors.neutralColor900,
                ),
          prefixIcon: buildFaIcon(widget.leftIcon),
          suffixIcon: buildFaIcon(widget.rightIcon),
          filled: true,
          fillColor: SeniorColors.neutralColor100,
          focusedBorder: InputBorder.none,
          errorStyle: TextStyle(
            fontSize: 0,
            height: 0,
          ),
          focusedErrorBorder: InputBorder.none,
          hintText: widget.hintText,
          counterStyle: TextStyle(
            fontSize: 0,
            height: 0,
          ),
          hintStyle: SeniorTypography.text(color: SeniorColors.neutralColor600),
        ),
      ),
    );
  }

  Widget buildHelper() {
    return Padding(
      padding: const EdgeInsets.only(
        top: 4.0,
        left: 14.0,
      ),
      child: Text(
        errorMsg ?? widget.helper ?? '',
        overflow: TextOverflow.ellipsis,
        style: SeniorTypography.helper(
          color: hasError
              ? SeniorColors.manchesterColorRed
              : SeniorColors.neutralColor700,
        ),
      ),
    );
  }

  Widget buildCounter() {
    return widget.maxLength == null
        ? SizedBox.shrink()
        : Padding(
            padding: const EdgeInsets.symmetric(
              horizontal: 14.0,
              vertical: 4.0,
            ),
            child: Text(
              '${widget.maxLength! - controller.text.length} ${widget.counterCaracterLabel ?? 'caracteres'}',
              style:
                  SeniorTypography.helper(color: SeniorColors.neutralColor700),
            ),
          );
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        buildTextField(),
        Row(
          children: [
            Expanded(child: buildHelper()),
            buildCounter(),
          ],
        ),
      ],
    );
  }
}

class _SeniorTextFieldPassword extends SeniorTextField {
  _SeniorTextFieldPassword({
    Key? key,
    String? label,
    IconData? leftIcon,
    String? helper,
    String? hintText,
    bool disabled = false,
    String? initialValue,
    FormFieldValidator<String>? validator,
    int? maxLength,
    bool autocorrect = false,
    bool readOnly = false,
    Function(String)? onChanged,
    Function()? onTap,
    Function()? onEditingComplete,
    Function(String?)? onFieldSubmitted,
    bool autofocus = false,
    TextEditingController? controller,
    TextInputAction? textInputAction,
  }) : super(
          key: key,
          label: label,
          leftIcon: leftIcon,
          helper: helper,
          hintText: hintText,
          disabled: disabled,
          initialValue: initialValue,
          validator: validator,
          maxLength: maxLength,
          autocorrect: autocorrect,
          readOnly: readOnly,
          onChanged: onChanged,
          onTap: onTap,
          onEditingComplete: onEditingComplete,
          onFieldSubmitted: onFieldSubmitted,
          autofocus: autofocus,
          controller: controller,
          keyboardType: TextInputType.visiblePassword,
          textInputAction: textInputAction,
        );

  @override
  _SeniorTextField createState() => _SeniorTextFieldPasswordState();
}

class _SeniorTextFieldPasswordState extends _SeniorTextField {
  bool obscureText = true;

  @override
  Widget buildTextField() {
    return AnimatedContainer(
      duration: const Duration(milliseconds: 200),
      color: hasFocus ? SeniorColors.neutralColor100 : Colors.transparent,
      child: Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.only(
            topLeft: Radius.circular(SeniorRadius.xbig),
            topRight: Radius.circular(SeniorRadius.xbig),
            bottomRight: Radius.circular(SeniorRadius.xsmall),
            bottomLeft: Radius.circular(SeniorRadius.xsmall),
          ),
          border: Border.all(
            color: hasFocus
                ? hasError
                    ? SeniorColors.manchesterColorRed
                    : SeniorColors.primaryColor
                : Colors.transparent,
            width: 2.0,
          ),
        ),
        child: TextFormField(
          key: widget.key,
          style: SeniorTypography.text(color: SeniorColors.neutralColor900),
          onChanged: onChanged,
          onTap: widget.onTap,
          validator: validate,
          focusNode: focusNode,
          minLines: widget.minLines,
          maxLines: widget.maxLines,
          autocorrect: widget.autocorrect,
          readOnly: widget.readOnly,
          onEditingComplete: widget.onEditingComplete,
          onFieldSubmitted: widget.onFieldSubmitted,
          autofocus: widget.autofocus,
          enabled: !widget.disabled,
          controller: controller,
          keyboardType: widget.keyboardType,
          textInputAction: widget.textInputAction ?? TextInputAction.done,
          obscureText: obscureText,
          decoration: InputDecoration(
            labelText: widget.label,
            labelStyle: hasFocus
                ? SeniorTypography.title(
                    color: hasError
                        ? SeniorColors.manchesterColorRed
                        : SeniorColors.primaryColor500,
                  )
                : SeniorTypography.text(
                    color: hasError
                        ? SeniorColors.manchesterColorRed
                        : SeniorColors.neutralColor900,
                  ),
            prefixIcon: buildFaIcon(widget.leftIcon),
            suffixIcon: buildPasswordEyeIcon(),
            filled: true,
            fillColor: SeniorColors.neutralColor100,
            focusedBorder: InputBorder.none,
            errorStyle: TextStyle(
              fontSize: 0,
              height: 0,
            ),
            counterStyle: TextStyle(
              fontSize: 0,
              height: 0,
            ),
            focusedErrorBorder: InputBorder.none,
            hintText: widget.hintText,
            hintStyle:
                SeniorTypography.text(color: SeniorColors.neutralColor600),
          ),
        ),
      ),
    );
  }

  Widget buildPasswordEyeIcon() {
    return Padding(
      padding: const EdgeInsets.symmetric(
        horizontal: 4.0,
      ),
      child: Material(
        child: InkWell(
          borderRadius: BorderRadius.circular(50.0),
          onTap: () {
            setState(() {
              obscureText = !obscureText;
            });
          },
          child: Ink(
            child: Padding(
              padding: const EdgeInsets.symmetric(
                horizontal: 8.0,
                vertical: 16.0,
              ),
              child: FaIcon(
                obscureText ? FontAwesomeIcons.eye : FontAwesomeIcons.eyeSlash,
                size: SeniorTypographyIconSize.CTA,
                color: SeniorColors.neutralColor900,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
