"""Scripts for Flutter CI"""

import sys
import getopt
from enum import Enum
from colorama import Fore

sys.path.append("./senior-ci")

from common import (
    exit_message,
    exec_command,
    exec_command_and_return,
    ExitCode,
    print_message,
    get_env_variable,
)

from common.extensions import Extensions
from common.sonar_helper import SonarScanner, SonarScannerType

from common.validate_issues import ProjectIssuesStatus


class Platform(Enum):
    ANDROID = 1
    IOS = 2


class FlutterAppsCI:
    def __init__(self, argv):
        try:
            self.opts, self.args = getopt.getopt(argv, "vbr:x", ["android", "ios", "flavor=", "target="])
        except getopt.GetoptError as err:
            exit_message(err)

        self.platform = None
        self.flavor = None
        self.target = None
        self._validade_options()
        self._configure_platform()
        self._configure_flavor()

    def exec_ci(self):
        for opt, arg in self.opts:
            if opt == "-v":
                self._validation()
            elif opt == "-b":
                self._build()
            elif opt == "-r":
                self._release(arg)
            elif opt == "-x":
                self._publish_sonar()
            else:
                exit_message("É preciso informar a opção referente a ação escolhida")

    def _validation(self):
        print_message("Executando validação de código...")

        extensions = Extensions()
        extensions.before_build()

        print_message("Executando validação do changelog...")

        command_result_changelog = exec_command_and_return(
            "bash senior-ci/scripts/validate_changelog.sh"
        )

        print(command_result_changelog.output)

        if command_result_changelog.exit_code == ExitCode.ERROR:
            exit_message("Validação de changelog finalizada com erro. Verifique")

        print_message("Executando análise estática...")

        command_result_analyse = exec_command_and_return(
            "fvm flutter analyze --no-pub --current-package --congratulate --preamble --no-fatal-infos --no-fatal-warnings --suppress-analytics --write=analyzer-output.txt"
        )

        print(command_result_analyse.output)

        if command_result_analyse.exit_code == ExitCode.ERROR:
            exit_message(
                "Análise estática finalizada com erro. Testes não serão executados."
            )

        self._exec_tests()

        extensions.after_build()

        sonar_scanner = SonarScanner(extensions, SonarScannerType.FLUTTER)

        sonar_scanner.preview()

        print(" ")
        print_message(
            "Verificação de código, execução de testes e analise estática finalizados com sucesso.",
            Fore.GREEN,
        )
        print(" ")

    def _build(self):
        print_message("Executando build...")
        pipeline_id = get_env_variable("CI_PIPELINE_ID")

        if not self.platform:
            exit_message("É preciso informar a plataforma que deseja realizar o build.")

        exec_command("fvm flutter clean")

        string_build = "fvm flutter build"
        if self.platform == Platform.ANDROID:
            string_build += " appbundle"
        else:
            string_build += " ios"
        string_build += f" --release --build-number={pipeline_id}"
        if self.flavor and self.target:
            string_build += f" --flavor={self.flavor} --target={self.target}"

        if self.platform == Platform.ANDROID:
            # Busca a Keystore das variáveis de ambiente e salva em um arquivo.
            exec_command("cat $SCI_ANDROID_KEYSTORE_JKS | base64 --decode > ./android/upload-keystore.jks")

        # Executa build
        command_result = exec_command_and_return(string_build)

        # Exibe o resultado do build
        print(command_result.output)

        if command_result.exit_code == ExitCode.SUCCESS:
            print(" ")
            print_message("Build finalizado com sucesso.", Fore.GREEN)
            print(" ")
        else:
            exit_message("Build finalizado com erro, verifique.")

    @staticmethod
    def _release(versioning_release):
        exec_command(f"bash senior-ci/mobile/flutter/scripts/release.sh -r {versioning_release}")

    @staticmethod
    def _exec_tests():
        print_message("Executando testes...")

        command_result_test = exec_command_and_return(
            "fvm flutter test --no-pub --reporter=expanded --coverage --suppress-analytics"
        )

        print(command_result_test.output)

        if command_result_test.exit_code == ExitCode.ERROR:
            exit_message("Testes finalizados com erro, verifique.")

    def _publish_sonar(self):
        self._exec_tests()

        sonar_scanner = SonarScanner(sonar_scanner_type=SonarScannerType.FLUTTER)

        sonar_scanner.publish()

    def _validade_options(self):
        len_opts = len(self.opts)

        if len_opts == 0:
            exit_message("É preciso informar as opções.")

    def _configure_platform(self):
        str_opts = str(self.opts)

        if "android" in str_opts and "ios" in str_opts:
            exit_message("Somente uma definição de plataforma é necessária.")
        elif "android" in str_opts:
            self.platform = Platform.ANDROID
            self.opts.remove(tuple(("--android", "")))
        elif "ios" in str_opts:
            self.platform = Platform.IOS
            self.opts.remove(tuple(("--ios", "")))

    def _configure_flavor(self):
        for opt, arg in self.opts:
            if opt == "--flavor":
                self.flavor = arg
            if opt == "--target":
                self.target = arg

        if self.flavor and not self.target:
            exit_message("Quando informado o flavor é necessário informar o target.")
        elif not self.flavor and self.target:
            exit_message("Quando informado o target é necessário informar o flavor.")
        elif self.flavor and self.target:
            self.opts.remove(tuple(("--flavor", self.flavor)))
            self.opts.remove(tuple(("--target", self.target)))
