"""Scripts for Flutter Libs CI"""

import sys
import getopt
from colorama import Fore

sys.path.append("./senior-ci")

from common import (
    exit_message,
    exec_command,
    exec_command_and_return,
    ExitCode,
    print_message,
)

from common.extensions import Extensions
from common.sonar_helper import SonarScanner, SonarScannerType
from common.validate_issues import ProjectIssuesStatus

class FlutterLibsCI:
    def __init__(self, argv):
        try:
            self.opts, self.args = getopt.getopt(argv, "vpr:x")
        except getopt.GetoptError as err:
            exit_message(err)

        self._validade_options()


    def exec_ci(self):
        for opt, arg in self.opts:
            if opt == "-v":
                self._validation()
            elif opt == "-p":
                self._publish_pubdev()
            elif opt == "-r":
                self._release(arg)
            elif opt == "-x":
                self._publish_sonar()
            else:
                exit_message("É preciso informar a opção referente a ação escolhida")


    def _validation(self):
        extensions = Extensions()
        extensions.before_build()

        self._exec_dart_analyze()
        self._exec_flutter_analyze()
        self._exec_tests()

        extensions.after_build()

        self._exec_sonar(extensions)


    def _publish_pubdev(self):
        print(" ")
        print_message(
            "Iniciando a publicação da nova versão da biblioteca...",
            Fore.YELLOW,
        )
        print(" ")

        exec_command("cat $SCI_PUB_DEV_CREDENTIALS > ~/.pub-cache/credentials.json")

        exec_command("fvm dart pub publish --force")

        print(" ")
        print_message(
            "Publicação da biblioteca finalizada.",
            Fore.GREEN,
        )


    def _publish_sonar(self):
        print(" ")
        print_message(
            "Iniciando a publicação das novas informações no Sonar...",
            Fore.YELLOW,
        )

        self._exec_tests()

        sonar_scanner = SonarScanner(sonar_scanner_type=SonarScannerType.FLUTTER)

        sonar_scanner.publish()


    @staticmethod
    def _exec_dart_analyze():
        print_message(
            "Iniciando a execução da validação da biblioteca...",
            Fore.YELLOW,
        )
        print(" ")

        command_result_analyse = exec_command_and_return(
            "fvm dart pub publish --dry-run"
        )

        print(command_result_analyse.output)

        if command_result_analyse.exit_code == ExitCode.SUCCESS:
            print(" ")
            print_message(
                "Validação da biblioteca finalizada com sucesso.",
                Fore.GREEN,
            )
        else:
            print(" ")
            exit_message("Validação da biblioteca finalizada com erro.")


    @staticmethod
    def _exec_flutter_analyze():
        print(" ")
        print_message(
            "Iniciando a execução da análise estática...",
            Fore.YELLOW,
        )
        print(" ")

        command_result_analyse = exec_command_and_return(
            "fvm flutter analyze --no-pub --current-package --congratulate --preamble --no-fatal-infos --no-fatal-warnings --suppress-analytics --write=analyzer-output.txt"
        )

        print(command_result_analyse.output)

        if command_result_analyse.exit_code == ExitCode.SUCCESS:
            print(" ")
            print_message(
                "Análise estática finalizada com sucesso.",
                Fore.GREEN,
            )
        else:
            print(" ")
            exit_message("Análise estática finalizada com erro.")


    @staticmethod
    def _exec_tests():
        print(" ")
        print_message(
            "Iniciando a execução dos testes unitários...",
            Fore.YELLOW,
        )
        print(" ")

        command_result_test = exec_command_and_return(
            "fvm flutter test --no-pub --reporter=expanded --coverage --suppress-analytics"
        )

        print(command_result_test.output)

        if command_result_test.exit_code == ExitCode.SUCCESS:
            print(" ")
            print_message(
                "Execução dos testes finalizada com sucesso.",
                Fore.GREEN,
            )
        else:
            print(" ")
            exit_message("Execução dos testes finalizada com erro.")


    @staticmethod
    def _release(versioning_release):
        exec_command(f"bash senior-ci/mobile/flutter/scripts/release.sh -r {versioning_release}")


    @staticmethod
    def _exec_sonar(extensions):
        print(" ")
        print_message(
            "Iniciando a execução do Sonar...",
            Fore.YELLOW,
        )
        print(" ")

        sonar_scanner = SonarScanner(extensions, SonarScannerType.FLUTTER)

        sonar_scanner.preview()

        print(" ")
        print_message(
            "Verificação de código, execução de testes e analise estática finalizados com sucesso.",
            Fore.GREEN,
        )
        print(" ")


    def _validade_options(self):
        len_opts = len(self.opts)

        if len_opts == 0:
            exit_message("É preciso informar as opções.")
