"""Module for common methods used on entire application"""

import os
import sys
import subprocess
import string
import random

from enum import Enum
from datetime import datetime
from colorama import Fore, Style


class ExitCode(Enum):
    SUCCESS = 0
    ERROR = 1


class CommandReturn:
    exit_code: ExitCode
    output: str

    @staticmethod
    def from_json(exit_code: int, command_output):
        command_return = CommandReturn()

        if exit_code == 0:
          command_return.exit_code = ExitCode.SUCCESS
        else:
          command_return.exit_code = ExitCode.ERROR

        command_return.output = command_output

        return command_return


def get_env_variable(variable_name):
    """Get the environment variable"""

    return os.getenv(variable_name)


def get_commit_tag_as_semver():
    """Get version from gitlab variable CI_COMMIT_TAG
    and format it to semver format (vX-Y-Z => X.Y.Z)
    """

    commit_tag = get_env_variable("CI_COMMIT_TAG")

    return commit_tag.replace("v", "").replace("-", ".") if commit_tag else None


def get_version_from_gitlab():
    """Get version from gitlab variables

    Get the tag if running from a tag or
    add the branch name and SNAPSHOT
    """

    version = get_commit_tag_as_semver()

    if not version:
        commit_ref_slug = get_env_variable("CI_COMMIT_REF_SLUG")

        version = f"{commit_ref_slug}-SNAPSHOT"

    return version


def print_message(message, fore_color=None, back_color=None):
    """Print a colored message"""

    fore_color_code = fore_color if fore_color else ""
    back_color_code = back_color if back_color else ""

    reset_style_code = Style.RESET_ALL if back_color or fore_color else ""

    print(f"{back_color_code}{fore_color_code}[CI] {message}{reset_style_code}")


def start_collapsible_section(header) -> str:
    """Print a message with the GitLab syntax to create a collapse section
    and return the id for use when close the section

    https://docs.gitlab.com/ee/ci/jobs/index.html#custom-collapsible-sections
    """

    section_id = "".join(random.choices(string.ascii_uppercase + string.digits, k=10))

    print(
        f"\033[0Ksection_start:{datetime.utcnow().strftime('%s')}:{section_id}\r\033[0K[CI] {header}"
    )

    return section_id


def end_collapsible_section(section_id):
    """Close the section"""

    print(
        f"\033[0Ksection_end:{datetime.utcnow().strftime('%s')}:{section_id}\r\033[0K"
    )


def exit_message(message):
    """Print a red error message and stop the execution"""

    print_message(message, Fore.LIGHTRED_EX)
    sys.exit(2)


def exec_command(command):
    """Print a command then execute"""

    print(command, flush=True)
    os.system(command)


def exec_command_without_print(command):
    """Execute a command without print it"""

    os.system(command)


def exec_command_and_return(command) -> CommandReturn:
    """Print a command then execute and return the output"""

    print(command, flush=True)

    process = subprocess.Popen(
        command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True
    )
    (output, error) = process.communicate()
    exit_code = process.wait()

    command_return = CommandReturn.from_json(exit_code, output.decode())

    return command_return
